#vider la mmoire
rm(list=ls())

#charger les donnes
setwd("D:/DataMining/Databases_for_mining/dataset_for_soft_dev_and_comparison/feature_selection/partial_dep_plot")
autos <- read.table("Autos_pdp.txt",header=TRUE,sep="\t",dec=".")
str(autos)

#ACP
acp <- princomp(autos[-1],cor=TRUE,scores=TRUE)
print(acp)

#index des variables selon le premier axe
idx1 <- order(acp$loadings[,1])
print(colnames(autos[-1])[idx1])

#corrlation entre les variables
library(corrplot)
corrplot(cor(autos[-1][idx1]))

#librairie mlr
library(mlr)

#tche pour le problme autos
autos.task <- mlr::makeClassifTask(data=autos,target="risky",positive="yes")

#filtrage selon l'AUC
filter.auc <- mlr::generateFilterValuesData(task=autos.task,method="auc")

#affichage
mlr::plotFilterValues(filter.auc)

#librairie mlr
library(caret)

#modlisation et mesure des performances en validation croise
m_reg <- caret::train(risky ~ ., data = autos, method="glm",trControl=trainControl(method='cv',number=10))
print(m_reg)

#coefficients du modle labor sur la totalit des donnes
print(summary(m_reg$finalModel))

#fonction pour le graphique PDP
my.pdp <- function(modele,X,variable,pas=20,use.logit=TRUE){
  #extrmes
  x_min <- min(X[,variable])
  x_max <- max(X[,variable])
  etendue <- (x_max - x_min)
  #squence de valeurs
  valeurs <- seq(from=x_min,to=x_max,length.out=pas)
  #vecteur de resultat
  res <- c()
  #pour chaque valeur
  for (v in valeurs){
    #copier les donnes
    XPrim <- X
    #remplacer
    XPrim[,variable] <- rep(v,nrow(X))
    #prediction
    p <- predict(modele,newdata=XPrim,type='prob')[,'yes']
    #si logit demand, transformer p en LOGIT
    if (use.logit == TRUE){
      p <- log(p/(1.0-p))
    }
    #rcupration de la moyenne des probas ou des logit
    mp <- mean(p)
    #stocker le rsultat
    res <- c(res,mp)
  }
  #titre du graphique
  titre.ord <- "P(yes/X)"
  #si logit demand
  if (use.logit == TRUE){
    titre.ord <- "LOGIT"
  }
  #graphique
  plot(valeurs,res,type='b',xlab=variable,ylab=titre.ord,main="Graphique PDP")
  #liste des valeurs et moyenne des probas
  return(list(values=valeurs,probas=res))
}

#calcul sur la variable 'wheel_base' -- LOGIT
my.pdp(m_reg,autos[-1],'wheel_base',use.logit=TRUE)

#calcul sur la variable 'wheel_base' -- probas
my.pdp(m_reg,autos[-1],'wheel_base',use.logit=FALSE)

#pour wb_bis
my.pdp(m_reg,autos[-1],'wb_bis',use.logit=FALSE)

#gradient boosting machine
m_gbm <- caret::train(risky ~ ., data = autos, method="gbm",trControl=trainControl(method='cv',number=10))
print(m_gbm)

#PDP pour wheel_base -- LOGIT
my.pdp(m_gbm,autos[-1],'wheel_base',pas=20,use.logit=TRUE)

#PDP pour wheel_base -- PROBAS
my.pdp(m_gbm,autos[-1],'wheel_base',pas=20,use.logit=FALSE)

#package iml
library(iml)

#objet predictor pour le modle gbm
obj_gbm <- iml::Predictor$new(m_gbm,data=autos,y="risky")
print(obj_gbm)

#calculer l'effet
pdp <- iml::FeatureEffect$new(obj_gbm,method="pdp",feature='wheel_base')
plot(pdp)

#dtail des valeurs
print(pdp$results)
